// Persistence of Vision Ray Tracer Scene Description File
// File: look_at.inc
// Vers: 3.1g
// Desc: Macro to use Point_At with objects 
// Date: 01.04.2002 
// Auth: Tim Nikias Wenclawiak 

/*File/Macro Description

The usage of Look_At is fairly easy. It is invoked with a
vector as parameter. This vector is the position where to look
at.
The macro assumes the object you are rotating is centered at
<0,0,0>, +y is the sky-vector and -z the original direction
the object is facing.

Example:
cone{<0,0,0>,2 <0,0,-2>,0 //A cone pointing towards -z
 pigment{rgb 1}
 rotate Look_At(5,8,20) //rotates the cone to look at the given position
 }

That's it! Hope you enjoy it...

For questions, mail me at tim.nikias@gmx.de
*/

#macro Look_At(_pos)

//X-angle-calculation
#local _X = <1,0,1>*_pos;
#if (vlength(_X) = 0)
 #local _xR = 0;
#else
 #local _xR_a=vdot(_pos,_X) / (vlength(_pos)*vlength(_X));
 #if (_xR_a>1) #local _xR_a=1; #end
 #if (_xR_a<-1) #local _xR_a=-1; #end
 #local _xR = degrees(acos( _xR_a));
#end

//Y-angle-calculation
#local _Y = <1,0,1>*_pos;
#if (vlength(_Y) = 0)
 #local _yR = 0; #else
 #local _yR = degrees(acos( vdot(-z,_Y) / (vlength(-z)*vlength(_Y)))); #end

//Pending on Value of X, Y-Rotation is applied positive or negative
#if (_pos.x > 0) #local _yR = -_yR; #end
//Pending on Value of Y, X-Rotation is applied positive or negative
#if (_pos.y < 0) #local _xR = -_xR; #end

//Now we place it all into a useful vector!
#declare _looker = <_xR, _yR, 0>;

//For Y-Parallel views we need specifications
#if (_pos.x = 0 & _pos.y < 0 & _pos.z =0)
 #local _looker = <-90,0,0>; #end
#if (_pos.x = 0 & _pos.y > 0 & _pos.z =0)
 #local _looker = <90,0,0>; #end

(_looker)

#end
