// Persistence of Vision Ray Tracer Scene Description File
// File: memo_array.inc
// Vers: 3.5
// Desc: 2 Macros to create/use a memo-array
// Date: 21.3.02
// Auth: Tim Nikias Wenclawiak

/*How to use
Declare an array with length equal to the amount of floats you
want to memorize, and declare an "index-pointer" (float-value=0).
Then use Init_Memo_Array to initialize the memo-array with 0 as
default value (you could also do that for yourself, but the usage
of the macro assures you have declared both, array and "index-pointer".

Then, whenever you need a new value, simply declare the value
with any name, but use the Do_Memo_Array-Macro with following
parameters:
Do_Memo_Array( Array-Identifier, "index-pointer", Max-Float-Value, Distance between new Floats) 

So, an example would be:

#declare Memorize=array[5] //memorizes last 5 entries
#declare Mem_Val=0;
Init_Memo_Array(Memorize,Mem_Val) //initializes array with 0

#declare Value=Do_Memo_Array(Memorize,Mem_Val, 100, 10); //returns new value


You can use the Do_Memo_Array-Macro anywhere, where floats
are allowed. That is what is returned, after the memorizing is
completed.

Optional:
You may declare Memo_Rand, the seed for the random-stream, like:
#declare Memo_Rand = seed( Some_Value );

But the include will use a default, if none is specified.

*/


//A default random-generator for the new values
#ifndef (Memo_Rand) #declare Memo_Rand=seed(260981); #end

/**Init_Memo_Array**/
//Generate empty array and return it, requires _Act_Index to be given as well
#macro Init_Memo_Array(_Array,_Act_Index)
 #local _Array_Length=dimension_size(_Array,1);
 #local A=0; #while (A<_Array_Length)
  #declare _Array[A]=0;
  #local A=A+1; #end
 #declare _Act_Index=0;
#end

/**Do_Memo_Array**/
//Generate new value, return that, and save in array
#macro Do_Memo_Array(_Array,_Act_Index,_Max_Value,_Distance)
 #local _Mem_Indices=dimension_size(_Array,1);
 //Generate new value
 #local _N=rand(Memo_Rand)*_Max_Value;
 #while (
  #local A=0; #while (A<_Mem_Indices)
    (_N>=_Array[A]-_Distance & _N<=_Array[A]+_Distance) |
    #local A=A+1; #end
    false )
  #local _N=rand(Memo_Rand)*_Max_Value;
 #end
 //Save newest value above oldest
 #declare _Array[_Act_Index]=_N;
 #declare _Act_Index=_Act_Index+1;
 #if (_Act_Index=_Mem_Indices) #declare _Act_Index=0; #end
 //Return new value
 (_N)
#end